package dragoscorlatescu.ro.pm_xsi0;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.speech.RecognizerIntent;
import android.speech.SpeechRecognizer;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends Activity {
    private static final String TAG = "PM X si 0";
    private static final int VOICE_RECOGNITION_REQUEST_CODE = 1001;
    private Button btnSpeak;
    private TextView next_symbol;
    private TextView input;
    private TextView dummy;

    final int RECIEVE_MESSAGE = 1;        // Status  for Handler
    private BluetoothAdapter btAdapter = null;
    private BluetoothSocket btSocket = null;
    private StringBuilder sb = new StringBuilder();
    private TicTacToe ttt;
    private static final String[] POSITIONS = {"top left", "top center", "top right", "center left",
                                                "center", "center right", "bottom left", "bottom center", "bottom right"};
    private static final String[] writeChars = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "a", "b", "c", "d", "e", "f", "g", "h", "i"};
    private int index = 0;
    private static ArrayList<String> positions = new ArrayList<String>(Arrays.asList(POSITIONS));

    private ConnectedThread mConnectedThread;

    // SPP UUID service
    private static final UUID MY_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");

    // MAC-address of Bluetooth module (you must edit this line)
    private static String address = "20:15:03:17:11:70";

    // 1 == X, 0 == 0
    private int typeToPlay = 1;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        ttt = new TicTacToe();
        index = 0;

        btAdapter = BluetoothAdapter.getDefaultAdapter();       // get Bluetooth adapter
        checkBTState();

        next_symbol = (TextView)findViewById(R.id.next_symbol);
        input = (TextView)findViewById(R.id.input);
        dummy = (TextView)findViewById(R.id.dummy);

        btnSpeak = (Button)findViewById(R.id.speak);
        checkVoiceRecognition();

        btnSpeak.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                speak();
            }
        });

        // Set up a pointer to the remote node using it's address.
        BluetoothDevice device = btAdapter.getRemoteDevice(address);

        // Two things are needed to make a connection:
        //   A MAC address, which we got above.
        //   A Service ID or UUID.  In this case we are using the
        //     UUID for SPP.
        try {
            btSocket = device.createRfcommSocketToServiceRecord(MY_UUID);
            btAdapter.cancelDiscovery();
            btSocket.connect();
            Log.d(TAG, "....Connection ok...");
        } catch (IOException e) {
            Log.d(TAG, "In onResume() and socket create failed: " + e.getMessage() + ".");
        }

        // Create a data stream so we can talk to server.
        Log.d(TAG, "...Create Socket...");

        mConnectedThread = new ConnectedThread(btSocket);
        mConnectedThread.start();

    }

    public void checkVoiceRecognition() {
        // Check if voice recognition is present
        PackageManager pm = getPackageManager();
        List<ResolveInfo> activities = pm.queryIntentActivities(new Intent(
                RecognizerIntent.ACTION_RECOGNIZE_SPEECH), 0);
        if (activities.size() == 0) {
            btnSpeak.setEnabled(false);
            btnSpeak.setText("Voice recognizer not present");
            Toast.makeText(this, "Voice recognizer not present",
                    Toast.LENGTH_SHORT).show();
        }
    }

    public void speak() {
        Intent intent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);

        // Specify the calling package to identify your application
        intent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, getClass()
                .getPackage().getName());

        // Display an hint to the user about what he should say.
        intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Position. Ex: top right, center left, bottom center, center");

        // Given an hint to the recognizer about what the user is going to say
        //There are two form of language model available
        //1.LANGUAGE_MODEL_WEB_SEARCH : For short phrases
        //2.LANGUAGE_MODEL_FREE_FORM  : If not sure about the words or phrases and its domain.
        intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL,
                RecognizerIntent.LANGUAGE_MODEL_FREE_FORM);

        int noOfMatches = 1;
        // Specify how many results you want to receive. The results will be
        // sorted where the first result is the one with higher confidence.
        intent.putExtra(RecognizerIntent.EXTRA_MAX_RESULTS, noOfMatches);
        //Start the Voice recognizer activity for the result.
        startActivityForResult(intent, VOICE_RECOGNITION_REQUEST_CODE);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == VOICE_RECOGNITION_REQUEST_CODE)

            //If Voice recognition is successful then it returns RESULT_OK
            if(resultCode == RESULT_OK) {

                ArrayList<String> textMatchList = data
                        .getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS);

                if (!textMatchList.isEmpty()) {
                    String str = textMatchList.get(0).toLowerCase();
                    dummy.setText(str);
                    if (positions.contains(str)) {
                        if (ttt.addElement(typeToPlay, positions.indexOf(str))) {
                            input.setText(str);
                            if (typeToPlay == 1) { // X
                                mConnectedThread.write(writeChars[positions.indexOf(str)]);
                            }
                            else {
                                mConnectedThread.write(writeChars[positions.indexOf(str) + 9]);
                            }
                            typeToPlay ^= 1;
                            if (typeToPlay == 0) {
                                next_symbol.setText("0");
                            }
                            else {
                                next_symbol.setText("X");
                            }
                            if (ttt.checkWin() == 1) {
                                input.setText("X Won!");
                                ttt.resetTable();
                                typeToPlay = 1;
                                next_symbol.setText("X");
                            }
                            else if (ttt.checkWin() == 0) {
                                input.setText("0 Won!");
                                ttt.resetTable();
                                typeToPlay = 1;
                                next_symbol.setText("X");
                            }
                            else if (ttt.checkWin() == -2) {
                                input.setText("Draw!");
                                ttt.resetTable();
                                typeToPlay = 1;
                                next_symbol.setText("X");
                            }
                        }
                        else {
                            input.setText("Error! Position already occupied!");
                        }
                    }
                    else {
                        input.setText("Error! Speak again!");
                    }
                }
                //Result code for various error.
            }else if(resultCode == RecognizerIntent.RESULT_AUDIO_ERROR){
                showToastMessage("Audio Error");
            }else if(resultCode == RecognizerIntent.RESULT_CLIENT_ERROR){
                showToastMessage("Client Error");
            }else if(resultCode == RecognizerIntent.RESULT_NETWORK_ERROR){
                showToastMessage("Network Error");
            }else if(resultCode == RecognizerIntent.RESULT_NO_MATCH){
                showToastMessage("No Match");
            }else if(resultCode == RecognizerIntent.RESULT_SERVER_ERROR){
                showToastMessage("Server Error");
            }
        super.onActivityResult(requestCode, resultCode, data);
    }

    /**
     * Helper method to show the toast message
     **/
    void showToastMessage(String message){
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();

        try     {
            btSocket.close();
        } catch (IOException e2) {
            Log.d(TAG, "In onPause() and failed to close socket." + e2.getMessage() + ".");
        }
    }

    @Override
    protected void onStop() {
        super.onStop();

        Log.d(TAG, "...In onStop()...");

        try     {
            btSocket.close();
        } catch (IOException e2) {
            Log.d(TAG, "In onStop() and failed to close socket." + e2.getMessage() + ".");
        }
    }

    private void checkBTState() {
        // Check for Bluetooth support and then check to make sure it is turned on
        // Emulator doesn't support Bluetooth and will return null
        if(btAdapter==null) {
            errorExit("Fatal Error", "Bluetooth not support");
        } else {
            if (btAdapter.isEnabled()) {
                Log.d(TAG, "...Bluetooth ON...");
            } else {
                //Prompt user to turn on Bluetooth
                Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                startActivityForResult(enableBtIntent, 1);
            }
        }
    }

    private class ConnectedThread extends Thread {
        private final InputStream mmInStream;
        private final OutputStream mmOutStream;
        private volatile ArrayList<String> messages = new ArrayList<String>();

        public ConnectedThread(BluetoothSocket socket) {
            InputStream tmpIn = null;
            OutputStream tmpOut = null;

            // Get the input and output streams, using temp objects because
            // member streams are final
            try {
                tmpIn = socket.getInputStream();
                tmpOut = socket.getOutputStream();
            } catch (IOException e) { }

            mmInStream = tmpIn;
            mmOutStream = tmpOut;
        }

        public void run() {

            // Keep listening to the InputStream until an exception occurs
            while (true) {
                if (messages.size() > 0) {
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            dummy.setText(dummy.getText() + "\nsending");
                        }
                    });
                    byte[] msgBuffer = messages.remove(0).getBytes();
                    try {
                        mmOutStream.write(msgBuffer);
                    } catch (IOException e) {
                        Log.d(TAG, "...Error data send: " + e.getMessage() + "...");
                    }
                }

                try {
                    Thread.sleep(20);
                }
                catch (Exception e) {
                }
            }
        }

        /* Call this from the main activity to send data to the remote device */
        public void write(String message) {
            messages.add(message);
        }
    }
}
